/**
 * UGENE - Integrated Bioinformatics Tools.
 * Copyright (C) 2008-2025 UniPro <ugene@unipro.ru>
 * http://ugene.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */

#include "TextContentFilterTask.h"

#include <U2Core/TextObject.h>
#include <U2Core/U2SafePoints.h>

#include "../ProjectFilterNames.h"

namespace U2 {

//////////////////////////////////////////////////////////////////////////
/// TextContentFilterTask
//////////////////////////////////////////////////////////////////////////

TextContentFilterTask::TextContentFilterTask(const ProjectTreeControllerModeSettings& settings, const QList<QPointer<Document>>& docs)
    : AbstractProjectFilterTask(settings, ProjectFilterNames::TEXT_CONTENT_FILTER_NAME, docs) {
}

bool TextContentFilterTask::filterAcceptsObject(GObject* obj) {
    auto textObject = qobject_cast<TextObject*>(obj);
    CHECK(textObject != nullptr, false);
    return settings.nameFilterAcceptsString(textObject->getText());
}

//////////////////////////////////////////////////////////////////////////
/// TextContentFilterTaskFactory
//////////////////////////////////////////////////////////////////////////

AbstractProjectFilterTask* TextContentFilterTaskFactory::createNewTask(const ProjectTreeControllerModeSettings& settings,
                                                                       const QList<QPointer<Document>>& docs) const {
    QList<QPointer<Document>> acceptedDocs = getAcceptedDocs(docs, {GObjectTypes::TEXT});
    return acceptedDocs.isEmpty() ? nullptr : new TextContentFilterTask(settings, acceptedDocs);
}

}  // namespace U2
