/**
 * UGENE - Integrated Bioinformatics Tools.
 * Copyright (C) 2008-2025 UniPro <ugene@unipro.ru>
 * http://ugene.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */

#pragma once

#include <U2Lang/LocalDomain.h>

namespace U2 {

namespace LocalWorkflow {

/**
 * more smart scheduler than a SimplestSequentialScheduler
 * seeks the nearest to the end of the workflow worker and ticks it
 * uses workflow's actors bindings graph
 */
class ElapsedTimeUpdater;
class LastReadyScheduler : public Scheduler {
public:
    LastReadyScheduler(Schema* sh);
    virtual ~LastReadyScheduler();

    // reimplemented from Worker
    void init() override;
    bool isReady() const override;
    Task* tick() override;
    bool isDone() const override;
    void cleanup() override;

    WorkerState getWorkerState(const ActorId& actor) override;
    Task* replayLastWorkerTick() override;
    bool cancelCurrentTaskIfAllowed() override;
    void makeOneTick(const ActorId& actor) override;

protected:
    WorkerState getWorkerState(const Actor* a) override;
    ActorId actorId() const;
    void measuredTick();

    QMap<int, QList<Actor*>> topologicSortedGraph;
    BaseWorker* lastWorker;
    bool canLastTaskBeCanceled;
    ActorId requestedActorForNextTick;
    ElapsedTimeUpdater* timeUpdater;
};

}  // namespace LocalWorkflow

}  // namespace U2
